from arkindex.documents.models import Element
from arkindex.project.tests import FixtureTestCase


class TestElementManager(FixtureTestCase):
    """Tests for ElementManager class"""

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.vol = Element.objects.get(name="Volume 2")
        cls.p1 = Element.objects.get(name="Volume 2, page 1r")
        cls.p2 = Element.objects.get(name="Volume 2, page 1v")
        cls.p3 = Element.objects.get(name="Volume 2, page 2r")
        cls.act = cls.corpus.elements.create(
            name='act',
            type=cls.corpus.types.get(slug='act'),
        )
        cls.act.add_parent(cls.p1)

    def test_get_ascending(self):
        # Use Page1, expect Volume
        ids = Element.objects.get_ascending(self.p1.id)
        self.assertCountEqual(ids, [self.vol])

    def test_get_ascending_empty(self):
        # Use Volume, expect empty list
        ids = Element.objects.get_ascending(self.vol.id)
        self.assertCountEqual(ids, [])

    def test_get_descending(self):
        # Use volume, expect all three pages and the act
        ids = Element.objects.get_descending(self.vol.id)
        self.assertCountEqual(ids, [self.p1, self.p2, self.p3, self.act])

    def test_get_descending_empty(self):
        # Use page 2, expect empty list
        ids = Element.objects.get_descending(self.p2.id)
        self.assertCountEqual(ids, [])

    def test_get_neighbors(self):
        with self.assertNumQueries(4):
            self.assertEqual(len(Element.objects.get_neighbors(self.p1, 0)), 1)
        with self.assertNumQueries(4):
            self.assertEqual(len(Element.objects.get_neighbors(self.p2)), 3)
        with self.assertNumQueries(4):
            self.assertEqual(len(Element.objects.get_neighbors(self.p3, 5)), 3)
