# Generated by Django 4.0.7 on 2022-10-14 13:27

from django.db import migrations, models


def disable_read_only_without_bucket(apps, schema_editor):
    """
    Turn off s3_read_only_bucket on servers without S3 buckets
    to ensure that the new check constraint is respected.
    """
    ImageServer = apps.get_model('images', 'ImageServer')
    ImageServer.objects.filter(s3_bucket__isnull=True).update(s3_read_only_bucket=False)


class Migration(migrations.Migration):

    dependencies = [
        ('images', '0009_remove_imageserver_validated'),
    ]

    operations = [
        migrations.RenameField(
            model_name='imageserver',
            old_name='read_only',
            new_name='s3_read_only_bucket',
        ),
        migrations.AlterField(
            model_name='imageserver',
            name='s3_read_only_bucket',
            field=models.BooleanField(
                default=False,
                help_text='Disable the extra image checks that are normally made for servers with S3 buckets.',
            ),
        ),
        migrations.RunPython(
            disable_read_only_without_bucket,
            reverse_code=migrations.RunPython.noop,
            elidable=True,
        ),
        migrations.AddConstraint(
            model_name='imageserver',
            constraint=models.CheckConstraint(
                check=models.Q(s3_read_only_bucket=False) | models.Q(s3_bucket__isnull=False),
                name='s3_read_only_bucket_requires_s3_bucket'
            ),
        ),
    ]
