# Generated by Django 4.0.5 on 2022-06-27 15:08

from django.db import migrations


def stringify_json(apps, schema_editor):
    if schema_editor.connection.vendor == "postgresql":
        # Postgres' jsonb type allows to stringify all payloads in one query per column
        schema_editor.execute(
            """
            UPDATE ponos_task
            SET env = (
                SELECT jsonb_object(array_agg(key), array_agg(value))
                FROM jsonb_each_text(env)
            )
            WHERE id in (
                SELECT id FROM ponos_task, jsonb_each(env) AS env_items
                WHERE jsonb_typeof(env_items.value) <> 'string'
            );
            """
        )
        schema_editor.execute(
            """
            UPDATE ponos_task
            SET extra_files = (
                SELECT jsonb_object(array_agg(key), array_agg(value))
                FROM jsonb_each_text(extra_files)
            )
            WHERE id in (
                SELECT id FROM ponos_task, jsonb_each(extra_files) AS extra_files_items
                WHERE jsonb_typeof(extra_files_items.value) <> 'string'
            );
            """
        )

    else:
        Task = apps.get_model("ponos", "Task")
        to_update = []
        for task in Task.objects.only("id", "env", "extra_files"):
            updated = False

            if task.env and not all(isinstance(value, str) for value in task.env):
                task.env = {key: str(value) for key, value in task.env.items()}
                updated = True

            if task.extra_files and not all(
                isinstance(value, str) for value in task.extra_files
            ):
                task.extra_files = {
                    key: str(value) for key, value in task.extra_files.items()
                }
                updated = True

            if updated:
                to_update.append(task)

        Task.objects.bulk_update(to_update, fields=["env", "extra_files"])


class Migration(migrations.Migration):

    dependencies = [
        ("ponos", "0031_emptyable_jsonfield"),
    ]

    operations = [
        migrations.RunPython(stringify_json, reverse_code=migrations.RunPython.noop),
    ]
