from django.db import migrations, models

import arkindex.project.fields


def comma_sep_char_to_list(apps, schema_editor):
    Task = apps.get_model("ponos", "Task")
    tasks_attrs = Task.objects.using('default').values_list("id", "old_tags")
    Task.objects.bulk_update(
        (
            Task(
                id=task_id,
                tags=old_tags.split(",") if old_tags else []
            ) for task_id, old_tags in tasks_attrs
        ),
        ['tags']
    )


def revert_comma_sep_char_to_list(apps, schema_editor):
    Task = apps.get_model("ponos", "Task")
    tasks_attrs = Task.objects.using('default').values_list("id", "tags")
    Task.objects.bulk_update(
        (
            Task(
                id=task_id,
                old_tags=",".join(tags)
            ) for task_id, tags in tasks_attrs
        ),
        ['old_tags']
    )


class Migration(migrations.Migration):

    dependencies = [
        ('ponos', '0034_alter_artifact_size'),
    ]

    operations = [
        migrations.RenameField(
            model_name="task",
            old_name="tags",
            new_name="old_tags",
        ),
        migrations.AddField(
            model_name='task',
            name='tags',
            field=arkindex.project.fields.ArrayField(
                base_field=models.CharField(max_length=250),
                blank=True,
                default=list,
                size=16,
            ),
        ),
        migrations.RunPython(
            comma_sep_char_to_list,
            reverse_code=revert_comma_sep_char_to_list,
        ),
        migrations.RemoveField(
            model_name='task',
            name='old_tags',
        ),
    ]
