# Generated by Django 4.0.4 on 2022-12-05 09:44

from django.db import migrations, models

from arkindex.process.models import WorkerActivityState


def set_started_on_started(apps, schema_editor):
    """
    In case this migration runs while some processes were running, some activities might be in a `started` state
    and we therefore need to set their start time before we can apply the new check constraint.
    We do know their start time though: their last update time will be the time where they were set to `started`.
    """
    WorkerActivity = apps.get_model('process', 'WorkerActivity')
    WorkerActivity.objects.filter(state=WorkerActivityState.Started).update(started=models.F('updated'))


class Migration(migrations.Migration):

    dependencies = [
        ('process', '0061_workeractivity_updated_triggers'),
    ]

    operations = [
        migrations.AddField(
            model_name='workeractivity',
            name='started',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.RunPython(
            set_started_on_started,
            reverse_code=migrations.RunPython.noop,
        ),
        migrations.AddConstraint(
            model_name='workeractivity',
            constraint=models.CheckConstraint(
                check=~models.Q(state=WorkerActivityState.Started) | models.Q(started__isnull=False),
                name='worker_activity_started_requires_started',
            ),
        ),
    ]
