# Generated by Django 4.1.5 on 2023-02-14 10:16

from django.db import migrations, models

from arkindex.process.models import ProcessMode


def remove_corpus_on_repository_processes(apps, schema_editor):
    """
    Processes that were ran on IIIF repositories had a corpus linked to them.
    This unsets the corpus on those as we now only support worker imports
    and repository processes should no longer have a corpus.
    """
    Process = apps.get_model('process', 'Process')
    Process.objects.filter(mode=ProcessMode.Repository).exclude(corpus=None).update(corpus=None)


class Migration(migrations.Migration):

    dependencies = [
        ('process', '0063_workeractivity_process_not_null'),
    ]

    operations = [
        migrations.RunPython(
            remove_corpus_on_repository_processes,
            reverse_code=migrations.RunPython.noop,
            elidable=True,
        ),
        migrations.AddConstraint(
            model_name='process',
            constraint=models.CheckConstraint(
                check=models.Q(mode__in=(ProcessMode.Local, ProcessMode.Repository)) ^ ~models.Q(corpus=None),
                name='check_process_corpus',
                violation_error_message='Local and repository processes cannot have a corpus, and other modes must have one set.',
            ),
        ),
        migrations.AddConstraint(
            model_name='process',
            constraint=models.CheckConstraint(
                check=~models.Q(mode=ProcessMode.Local) | models.Q(workflow=None),
                name='local_process_no_workflow',
                violation_error_message='Local processes cannot be started.',
            ),
        ),
        migrations.AddConstraint(
            model_name='process',
            constraint=models.UniqueConstraint(
                models.F('creator'),
                condition=models.Q(mode=ProcessMode.Local),
                name='unique_local_process',
                violation_error_message='Only one local process is allowed per user.',
            ),
        ),
    ]
