from uuid import uuid4

from arkindex.documents.models import Classification, Element, Entity, MetaData, Transcription, TranscriptionEntity
from arkindex.ponos.models import Artifact
from arkindex.process.models import CorpusWorkerVersion, Repository, WorkerType, WorkerVersionState
from arkindex.project.tests import FixtureTestCase


class TestManagers(FixtureTestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.repo = Repository.objects.get(url='http://my_repo.fake/workers/worker')
        cls.revision = cls.repo.revisions.first()
        cls.artifact = Artifact.objects.get()
        # The fixtures have two worker versions, only one of them is used in existing elements
        cls.recognizer = cls.repo.workers.get(slug='reco').versions.get()

        # Retrieve one workerType
        cls.worker_type = WorkerType.objects.get(slug='recognizer')

    def _make_worker_version(self):
        return self.revision.versions.create(
            worker=self.repo.workers.create(slug=str(uuid4()), type=self.worker_type),
            configuration={},
            state=WorkerVersionState.Available,
            docker_image=Artifact.objects.first(),
        )

    def test_corpus_worker_version_rebuild(self):
        # Assign a different worker version for each ML result to get a lot of versions
        querysets = [
            Element.objects.filter(worker_version_id=None),
            Transcription.objects.filter(worker_version_id=None),
            TranscriptionEntity.objects.filter(worker_version_id=None),
            Entity.objects.filter(worker_version_id=None),
            Classification.objects.filter(worker_version_id=None),
            MetaData.objects.filter(worker_version_id=None),
        ]
        versions = [self.recognizer]
        for queryset in querysets:
            for obj in queryset:
                version = self._make_worker_version()
                versions.append(version)
                obj.worker_version = version
                obj.save()

        self.assertFalse(self.corpus.worker_versions.exists())
        CorpusWorkerVersion.objects.rebuild()
        self.assertCountEqual(self.corpus.worker_versions.all(), versions)
